/*****************************************************************************
 * Copyright (C) 2019-2020
 *
 * Michiel Broek <mbroek at mbse dot eu>
 *
 * This file is part of BMS
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * Brewery Management System istributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ThermFerm; see the file COPYING.  If not, write to the Free
 * Software Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 *****************************************************************************/

function createAbortElements() {
 $('#eventWindow').jqxWindow({
  theme: theme,
  position: { x: 440, y: 210 },
  width: 400,
  height: 200,
  resizable: false,
  isModal: true,
  modalOpacity: 0.4,
  okButton: $('#delOk'),
  cancelButton: $('#delCancel'),
  initContent: function() {
   $('#delOk').jqxButton({ template: 'danger', width: '65px', theme: theme });
   $('#delCancel').jqxButton({ template: 'success', width: '65px', theme: theme });
   $('#delCancel').focus();
  }
 });
 $('#eventWindow').jqxWindow('hide');
}


$(document).ready(function() {

 var record = {},
 blank = {},
 ppayload = '',
 yl = 12, // Normal yeast temp range
 yh = 24,

 productSource = {
  datatype: 'json',
  cache: false,
  datafields: [
   { name: 'code', type: 'string' },
   { name: 'name', type: 'string' },
   { name: 'uuid', type: 'string' },
   { name: 'stage', type: 'int' },
   { name: 'yeast_lo', type: 'float' },
   { name: 'yeast_hi', type: 'float' }
  ],
  id: 'code',
  url: 'includes/db_product.php?select=ferment'
 },
 productlist = new $.jqx.dataAdapter(productSource, {
  beforeLoadComplete: function(records) {
   var row, i, data = new Array();
   // Create a dummy beer on top to store in idle fermenters.
   blank['code'] = 'Free';  // Will override this later.
   blank['name'] = 'Dummy';
   blank['uuid'] = '66ecccbf-e942-4a35-af49-8b02314561a5';
   blank['stage'] = 10;
   blank['yeast_lo'] = 12.0;
   blank['yeast_hi'] = 24.0;
   data.push(blank);
   for (i = 0; i < records.length; i++) {
    row = records[i];
    data.push(row);
   }
   return data;
  },
  loadError: function(jqXHR, status, error) { console.log(status + ' ' + error); },
 }),
 profileSource = {
  datatype: 'json',
  cache: false,
  datafields: [
   { name: 'record', type: 'int' },
   { name: 'uuid', type: 'string' },
   { name: 'name', type: 'string' },
   { name: 'inittemp_lo', type: 'float' },
   { name: 'inittemp_hi', type: 'float' },
   { name: 'fridgemode', type: 'int' },
   { name: 'totalsteps', type: 'int' },
   { name: 'duration', type: 'int' },
   { name: 'steps', type: 'array' }
  ],
  id: 'record',
  url: 'includes/db_profile_fermentation.php'
 },
 profilelist = new $.jqx.dataAdapter(profileSource, {
  beforeLoadComplete: function(records) {
   var data = new Array(),
   empty = {}, i, row;
   // Create a dummy profile on top of the list.
   empty['record'] = -1;
   empty['uuid'] = '';
   empty['name'] = 'Wis profiel';
   empty['inittemp_lo'] = 20.0;
   empty['inittemp_hi'] = 20.2;
   empty['fridgemode'] = 0;
   empty['totalsteps'] = 0;
   empty['duration'] = 0;
   empty['steps'] = '[]';
   data.push(empty);
   for (i = 0; i < records.length; i++) {
    row = records[i];
    data.push(row);
   }
   return data;
  },
  loadError: function(jqXHR, status, error) {
   $('#err').text(status + ' ' + error);
  },
 }),
 gaugeoptions = {
  min: 0, max: 45, width: 375, height: 375,
  ranges: [{ startValue: 0, endValue: yl, style: { fill: '#3399FF', stroke: '#3399FF' }, endWidth: 10, startWidth: 10 },
           { startValue: yl, endValue: yh, style: { fill: '#00CC33', stroke: '#00CC33' }, endWidth: 10, startWidth: 10 },
           { startValue: yh, endValue: 45, style: { fill: '#FC6A6A', stroke: '#FC6A6A' }, endWidth: 10, startWidth: 10 }],
  ticksMinor: { interval: 1, size: '5%' },
  ticksMajor: { interval: 5, size: '9%' },
  labels: { interval: 5 },
  style: { fill: '#eeeeee', stroke: '#666666' },
  value: 0,
  colorScheme: 'scheme05'
 },
 gaugeSmalloptions = {
  min: -15, max: 25, width: 190, height: 190,
  ranges: [{ startValue: -15, endValue: 0, startWidth: 5, endWidth: 5, style: { fill: '#3399FF', stroke: '#3399FF' }},
           { startValue: 0, endValue: 10, startWidth: 5, endWidth: 5, style: { fill: '#00CC33', stroke: '#00CC33' }},
           { startValue: 10, endValue: 25, startWidth: 5, endWidth: 5, style: { fill: '#FC6A6A', stroke: '#FC6A6A' }}],
  ticksMinor: { interval: 1, size: '5%' },
  ticksMajor: { interval: 5, size: '9%' },
  labels: { interval: 5 },
  style: { fill: '#eeeeee', stroke: '#666666' },
  value: 0,
  colorScheme: 'scheme05',
  caption: { value: 'Koeler', position: 'bottom', offset: [0, 10] }
 },
 switchoptions = {
  height: 68,
  width: 35,
  onLabel: 'AAN',
  offLabel: 'UIT',
  theme: theme,
  thumbSize: '50%',
  orientation: 'vertical'
 },
 targetoptions = { inputMode: 'simple', theme: theme, width: 70, min: 0, max: 45, decimalDigits: 1, spinButtons: true },
 url = 'getfermenter.php?uuid="' + my_uuid + '"',
 source = {
  datatype: 'json',
  datafields: [
   { name: 'record', type: 'int' },
   { name: 'uuid', type: 'string' },
   { name: 'alias', type: 'string' },
   { name: 'node', type: 'string' },
   { name: 'online', type: 'int' },
   { name: 'beercode', type: 'string' },
   { name: 'beername', type: 'string' },
   { name: 'air_state', type: 'string' },
   { name: 'air_temperature', type: 'float' },
   { name: 'beer_state', type: 'string' },
   { name: 'beer_temperature', type: 'float' },
   { name: 'chiller_state', type: 'string' },
   { name: 'chiller_temperature', type: 'float' },
   { name: 'heater_state', type: 'int' },
   { name: 'heater_usage', type: 'int' },
   { name: 'cooler_state', type: 'int' },
   { name: 'cooler_usage', type: 'int' },
   { name: 'fan_state', type: 'int' },
   { name: 'fan_usage', type: 'int' },
   { name: 'light_address', type: 'string' },
   { name: 'light_state', type: 'int' },
   { name: 'light_usage', type: 'int' },
   { name: 'door_address', type: 'string' },
   { name: 'door_state', type: 'int' },
   { name: 'psu_address', type: 'string' },
   { name: 'psu_state', type: 'int' },
   { name: 'mode', type: 'string' },
   { name: 'alarm', type: 'int' },
   { name: 'setpoint_high', type: 'float' },
   { name: 'setpoint_low', type: 'float' },
   { name: 'profile_uuid', type: 'string' },
   { name: 'profile_name', type: 'string' },
   { name: 'profile_state', type: 'string' },
   { name: 'profile_percent', type: 'int' },
   { name: 'profile_inittemp_high', type: 'float' },
   { name: 'profile_inittemp_low', type: 'float' },
   { name: 'profile_steps', type: 'string' },
   { name: 'stage', type: 'string' },
   { name: 'beeruuid', type: 'string' },
   { name: 'yeast_lo', type: 'float' },
   { name: 'yeast_hi', type: 'float' },
   { name: 'webcam_url', type: 'string' },
   { name: 'webcam_light', type: 'int' }
  ],
  id: 'record',
  url: url
 },
 dataAdapter = new $.jqx.dataAdapter(source, {
  loadComplete: function(records) {
   record = dataAdapter.records[0];
   updateScreen();
   blank['name'] = record.alias;
   blank['code'] = record.alias.toUpperCase();
   blank['uuid'] = record.uuid;
  }
 });

 function updateScreen() {
   $('#info_uuid').html(record.uuid);
   $('#info_system').html(record.node + '/' + record.alias);

   if (record.online) {
    $('#info_online').html('On-line');
    $('#info_beer').html(record.beercode + ' - ' + record.beername);
    $('#info_mode').jqxDropDownList('selectItem', record.mode);
    $('#info_stage').jqxDropDownList('selectItem', record.stage);
    if (record.door_address) {
     if (record.door_state != '0') {
      $('#fermenter_doorled').html('<div class="LEDyellow_on"></div>Door');
     } else {
      $('#fermenter_doorled').html('<div class="LEDyellow_off"></div>Door');
     }
    }
    if (record.light_address) {
     if (record.light_state != '0') {
      $('#fermenter_lightled').html('<div class="LEDyellow_on"></div>Light');
     } else {
      $('#fermenter_lightled').html('<div class="LEDyellow_off"></div>Light');
     }
    }
    if (record.mode != 'OFF') {
     $('#fermenter_powerled').html('<div class="LEDblue_on"></div>Power');
     $('#select_beer').jqxDropDownList({ disabled: true });
     $('#select_beer').jqxDropDownList('clearSelection');
     $('#select_beer').hide();
    } else {
     $('#fermenter_powerled').html('<div class="LEDblue_off"></div>Power');
     $('#select_beer').show();
     $('#select_beer').jqxDropDownList({ disabled: false });
    }
    if (record.alarm != '0') {
     $('#fermenter_alarmled').html('<div class="LEDred_on"></div>Alarm');
    } else {
     $('#fermenter_alarmled').html('<div class="LEDred_off"></div>Alarm');
    }

    $('#target_lo').val(record.setpoint_low);
    $('#target_hi').val(record.setpoint_high);
    if ((record.mode == 'FRIDGE') || (record.mode == 'BEER')) {
     $('#target_lo').jqxNumberInput({ readOnly: false, Width: 70, spinButtons: true });
     $('#target_hi').jqxNumberInput({ readOnly: false, Width: 70, spinButtons: true });
    } else {
     $('#target_lo').jqxNumberInput({ readOnly: true, Width: 50, spinButtons: false });
     $('#target_hi').jqxNumberInput({ readOnly: true, Width: 50, spinButtons: false });
    }

    $('.f_control_leds').show();
    if (record.heater_state != '0') {
     $('#fermenter_led1').html('<div class="LEDgreen_on"></div>Heat');
    } else {
     $('#fermenter_led1').html('<div class="LEDgreen_off"></div>Heat');
    }
    if (record.cooler_state != '0') {
     $('#fermenter_led2').html('<div class="LEDgreen_on"></div>Cool');
    } else {
     $('#fermenter_led2').html('<div class="LEDgreen_off"></div>Cool');
    }
    if (record.fan_state != '0') {
     $('#fermenter_led3').html('<div class="LEDgreen_on"></div>Fan');
    } else {
     $('#fermenter_led3').html('<div class="LEDgreen_off"></div>Fan');
    }

    if (record.mode == 'NONE') {
     $('.f_control_switches').show();
    } else {
     $('.f_control_switches').hide();
    }
    if ((record.heater_state != '0') != $('#fermenter_toggle1').jqxSwitchButton('val'))
     $('#fermenter_toggle1').val(record.heater_state != '0');
    if ((record.cooler_state != '0') != $('#fermenter_toggle2').jqxSwitchButton('val'))
     $('#fermenter_toggle2').val(record.cooler_state != '0');
    if ((record.fan_state != '0') != $('#fermenter_toggle3').jqxSwitchButton('val'))
     $('#fermenter_toggle3').val(record.fan_state != '0');

    $('#info_profile').html(record.profile_name);
    if (record.profile_name == '')
     $('#info_mode').jqxDropDownList('disableItem', 'PROFILE');
    else
     $('#info_mode').jqxDropDownList('enableItem', 'PROFILE');

    if (record.mode == 'PROFILE') {
     if (record.profile_state == 'OFF') {
      $('#select_profile').show();
      $('#select_profile').jqxDropDownList({ disabled: false });
      $('#info_mode').jqxDropDownList({ disabled: false });
      $('#Profile1').jqxButton({ template: 'success', value: 'Starten' });
      $('#Profile1').show();
      $('#Profile2').hide();
      $('#status_profile').html('');
     } else if (record.profile_state == 'RUN') {
      $('#select_profile').jqxDropDownList({ disabled: true });
      $('#select_profile').hide();
      $('#info_mode').jqxDropDownList({ disabled: true });
      $('#Profile1').jqxButton({ template: 'danger', value: 'Afbreken' });
      $('#Profile2').jqxButton({ template: 'primary', value: 'Pauze' });
      $('#Profile1').show();
      $('#Profile2').show();
      $('#status_profile').html('Profiel actief, ' + record.profile_percent + '% gereed');
     } else if (record.profile_state == 'PAUSE') {
      $('#select_profile').jqxDropDownList({ disabled: true });
      $('#select_profile').hide();
      $('#info_mode').jqxDropDownList({ disabled: true });
      $('#Profile1').jqxButton({ template: 'danger', value: 'Afbreken' });
      $('#Profile2').jqxButton({ template: 'success', value: 'Doorgaan' });
      $('#Profile1').show();
      $('#Profile2').show();
      $('#status_profile').html('Profiel pauze, ' + record.profile_percent + '% gereed');
     } else if (record.profile_state == 'DONE') {
      $('#select_profile').jqxDropDownList({ disabled: true });
      $('#select_profile').hide();
      $('#info_mode').jqxDropDownList({ disabled: true });
      $('#Profile1').jqxButton({ template: 'primary', value: 'Profiel Ok' });
      $('#Profile1').show();
      $('#Profile2').hide();
      $('#status_profile').html('Profiel is gereed');
     }
    } else {
     $('#select_profile').show();
     $('#select_profile').jqxDropDownList({ disabled: false });
     $('#info_mode').jqxDropDownList({ disabled: false });
     $('#Profile1').hide();
     $('#Profile2').hide();
     $('#status_profile').html('');
    }

    if (record.webcam_url != '') {
     $('#Camera').show();
    } else {
     $('#Camera').hide();
    }

    var yl = record.yeast_lo;
    var yh = record.yeast_hi;
    var range = { ranges: [{ startValue: 0, endValue: yl, style: { fill: '#3399FF', stroke: '#3399FF' }, endWidth: 10, startWidth: 10 },
                      { startValue: yl, endValue: yh, style: { fill: '#00CC33', stroke: '#00CC33' }, endWidth: 10, startWidth: 10 },
                      { startValue: yh, endValue: 45, style: { fill: '#FC6A6A', stroke: '#FC6A6A' }, endWidth: 10, startWidth: 10 }]};
    $('#gaugeContainer_air').jqxGauge(range);
    $('#gaugeContainer_beer').jqxGauge(range);

    if (record.air_temperature !== undefined) {
     $('#gaugeContainer_air').jqxGauge({ caption: { value: 'Lucht: ' + record.air_temperature.toFixed(3) }});
     $('#gaugeContainer_air').jqxGauge({ value: record.air_temperature });
    }
    if (record.air_state == 'OK') {
     $('#gaugeContainer_air').jqxGauge({ disabled: false });
    } else {
     $('#gaugeContainer_air').jqxGauge({ disabled: true });
    }
    if (record.beer_temperature !== undefined) {
     $('#gaugeContainer_beer').jqxGauge({ caption: { value: 'Bier: ' + record.beer_temperature.toFixed(3) }});
     $('#gaugeContainer_beer').jqxGauge({ value: record.beer_temperature });
    }
    if (record.beer_state == 'OK') {
     $('#gaugeContainer_beer').jqxGauge({ disabled: false });
    } else {
     $('#gaugeContainer_beer').jqxGauge({ disabled: true });
    }
    if (record.chiller_temperature !== undefined) {
     $('#gaugeContainer_chiller').jqxGauge({ value: record.chiller_temperature });
    }
    if (record.chiller_state == 'OK') {
     $('#gaugeContainer_chiller').jqxGauge({ disabled: false });
    } else {
     $('#gaugeContainer_chiller').jqxGauge({ disabled: true });
    }
   } else { // offline
    $('#info_online').html('Off-line');
    $('#info_beer').html('');
    $('#info_mode').hide();
    $('#info_stage').hide();
    $('#select_beer').hide();
    $('#select_profile').hide();
    $('.f_display,.f_control_switches,.f_control_leds').hide();
    $('#fermenter_powerled').html('<div class="LEDblue_off"></div>Power');
    $('#fermenter_alarmled').html('<div class="LEDred_on"></div>Alarm');
    $('#gaugeContainer_air').jqxGauge({ disabled: true });
    $('#gaugeContainer_beer').jqxGauge({ disabled: true });
    $('#gaugeContainer_chiller').jqxGauge({ disabled: true });
    $('#Camera').hide();
   }
 }

 $('#select_beer').jqxDropDownList({
  placeHolder: 'Kies bier:',
  theme: theme,
  source: productlist,
  displayMember: 'code',
  width: 150,
  height: 24,
  dropDownWidth: 500,
  autoDropDownHeight: true,
  renderer: function(index, label, value) {
   var datarecord = productlist.records[index];
   return datarecord.code + ' - ' + datarecord.name;
  }
 });
 $('#select_profile').jqxDropDownList({
  placeHolder: 'Kies profiel:',
  theme: theme,
  source: profilelist,
  displayMember: 'name',
  width: 150,
  height: 24,
  dropDownWidth: 500,
  autoDropDownHeight: true,
 });

 $('#gaugeContainer_air').jqxGauge(gaugeoptions);
 $('#gaugeContainer_air').jqxGauge({ caption: { value: 'Lucht: 00.000' }});
 $('#gaugeContainer_beer').jqxGauge(gaugeoptions);
 $('#gaugeContainer_beer').jqxGauge({ caption: { value: 'Bier: 00.000' }});
 $('#gaugeContainer_chiller').jqxGauge(gaugeSmalloptions);

 $('#fermenter_toggle1').jqxSwitchButton(switchoptions);
 $('#fermenter_toggle2').jqxSwitchButton(switchoptions);
 $('#fermenter_toggle3').jqxSwitchButton(switchoptions);

 srcMode = ['OFF', 'NONE', 'FRIDGE', 'BEER', 'PROFILE'];
 srcStage = ['PRIMARY', 'SECONDARY', 'TERTIARY', 'CARBONATION'];
 $('#info_mode').jqxDropDownList({ theme: theme, source: srcMode, width: 100, height: 24, dropDownHeight: 156 });
 $('#info_stage').jqxDropDownList({ theme: theme, source: srcStage, width: 150, height: 24, dropDownHeight: 125 });

 $('#target_lo').jqxNumberInput(targetoptions);
 $('#target_hi').jqxNumberInput(targetoptions);

 $('#Profile1').jqxButton({ template: 'info', width: '150px', height: 24, theme: theme });
 $('#Profile2').jqxButton({ template: 'info', width: '150px', height: 24, theme: theme });
 $('#Profile1').hide(); // Hide these until they are needed.
 $('#Profile2').hide();

 // Get the data immediatly and then at regular intervals to refresh.
 dataAdapter.dataBind();

 $('#info_mode').on('select', function(event) {
  if (event.args && event.args.item.value != record.mode) {
   record.mode = event.args.item.value;
   console.log('set mode ' + record.mode);
   var msg = '{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias + '","mode":"' + record.mode + '"}';
   websocket.send(msg);
  }
 });
 $('#info_stage').on('select', function(event) {
  if (event.args && event.args.item.value != record.stage) {
   record.stage = event.args.item.value;
   console.log('set stage ' + record.stage);
   var msg = '{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias + '","stage":"' + record.stage + '"}';
   websocket.send(msg);
  }
 });
 $('#select_beer').on('select', function(event) {
  if (event.args) {
   var index = event.args.index,
   datarecord = productlist.records[index];
   record.beercode = datarecord.code;
   record.beername = datarecord.name;
   record.beeruuid = datarecord.uuid;
   record.yeast_lo = datarecord.yeast_lo;
   record.yeast_hi = datarecord.yeast_hi;
   console.log('set beer ' + record.beercode + ' ' + record.beername);
   var msg = '{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias +
             '","beeruuid":"' + record.beeruuid + '","beercode":"' + record.beercode + '","beername":"' + record.beername +
             '","yeast_lo":' + record.yeast_lo + ',"yeast_hi":' + record.yeast_hi + '}';
   websocket.send(msg);
  }
 });
 $('#select_profile').on('select', function(event) {
  if (event.args) {
   var index = event.args.index,
   datarecord = profilelist.records[index],
   row, i;
   if (datarecord.record == -1) {
    ppayload = '","profile":null}';
   } else {
    ppayload = '","profile":{"uuid":"' + datarecord.uuid + '","name":"' + datarecord.name + '",';
    ppayload += '"inittemp":{"low":' + datarecord.inittemp_lo + ',"high":' + datarecord.inittemp_hi + '},';
    ppayload += '"fridgemode":' + datarecord.fridgemode + ',"steps":[';
    for (i = 0; i < datarecord.steps.length; i++) {
     row = datarecord.steps[i];
     if (i > 0)
      ppayload += ',';
     ppayload += '{"steptime":' + row['steptime'] + ',"resttime":' + row['resttime'];
     ppayload += ',"target_lo":' + row['target_lo'] + ',"target_hi":' + row['target_hi'];
     ppayload += ',"fridgemode":' + row['fridgemode'] + ',"name":"' + row['name'] + '"}';
    }
    ppayload += ']}}';
   }
   var msg = '{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias + ppayload;
   websocket.send(msg);
  }
 });

 $('#target_lo').on('change', function(event) {
  record.setpoint_low = parseFloat(event.args.value);
  // Keep the high target above the low.
  if (record.setpoint_low > record.setpoint_high) {
   record.setpoint_high = record.setpoint_low;
   $('#target_hi').val(record.setpoint_high);
  }
  console.log('set setpoints ' + record.setpoint_low + ' ' + record.setpoint_high);
  websocket.send('{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias +
            '","setpoint_low":' + record.setpoint_low + ',"setpoint_high":' + record.setpoint_high + '}');
 });
 $('#target_hi').on('change', function(event) {
  record.setpoint_high = parseFloat(event.args.value);
  // Keep the low target below the high.
  if (record.setpoint_high < record.setpoint_low) {
   record.setpoint_low = record.setpoint_high;
   $('#target_lo').val(record.setpoint_low);
  }
  console.log('set setpoints ' + record.setpoint_low + ' ' + record.setpoint_high);
  websocket.send('{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias +
            '","setpoint_low":' + record.setpoint_low + ',"setpoint_high":' + record.setpoint_high + '}');
 });

 $('#fermenter_toggle1').on('checked', function(event) {
  if (record.mode == 'NONE' && record.heater_state != 0) {
   console.log('set heater ' + $("#fermenter_toggle1").jqxSwitchButton('val'));
   websocket.send('{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias + '","heater_state":0}');
  }
 });
 $('#fermenter_toggle1').on('unchecked', function(event) {
  if (record.mode == 'NONE' && record.heater_state == 0) {
   console.log('set heater ' + $("#fermenter_toggle1").jqxSwitchButton('val'));
   websocket.send('{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias + '","heater_state":100,"cooler_state":0}');
  }
 });
 $('#fermenter_toggle2').on('checked', function(event) {
  if (record.mode == 'NONE' && record.cooler_state != 0) {
   console.log('set cooler ' + $("#fermenter_toggle2").jqxSwitchButton('val'));
   websocket.send('{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias + '","cooler_state":0}');
  }
 });
 $('#fermenter_toggle2').on('unchecked', function(event) {
  if (record.mode == 'NONE' & record.cooler_state == 0) {
   console.log('set cooler ' + $("#fermenter_toggle2").jqxSwitchButton('val'));
   websocket.send('{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias + '","cooler_state":100,"heater_state":0}');
  }
 });
 $('#fermenter_toggle3').on('checked', function(event) {
  if (record.mode == 'NONE' && record.fan_state != 0) {
   websocket.send('{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias + '","fan_state":0}');
  }
 });
 $('#fermenter_toggle3').on('unchecked', function(event) {
  if (record.mode == 'NONE' && record.fan_state == 0) {
   websocket.send('{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias + '","fan_state":100}');
  }
 });
 $('#Profile1').click(function() {
  if (record.mode == 'PROFILE') {
   if (record.profile_state == 'OFF') {
    websocket.send('{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias + '","profile":{"command":"start"}}');
   } else if ((record.profile_state == 'RUN') || (record.profile_state == 'PAUSE')) {
    // Open a popup to confirm this action.
    $('#eventWindow').jqxWindow('open');
    $('#delOk').click(function() {
     websocket.send('{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias + '","profile":{"command":"abort"}}');
    });
   } else if (record.profile_state == 'DONE') {
    websocket.send('{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias + '","profile":{"command":"done"}}');
   }
  }
 });
 $('#Profile2').click(function() {
  if (record.mode == 'PROFILE') {
   if ((record.profile_state == 'RUN') || (record.profile_state == 'PAUSE')) {
    websocket.send('{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias + '","profile":{"command":"pause"}}');
   }
  }
 });

 // The chart button.
 $('#FLog').jqxButton({ template: 'primary', width: '150px', theme: theme });
 $('#FLog').click(function() {
  window.open('log_fermentation.php?code=' + record.beercode + '&name=' + record.beername);
 });
 $('#Camera').jqxButton({ template: 'primary', width: '150px', theme: theme });
 $('#Camera').click(function() {
//  record.light_state = 100;
  websocket.send('{"device":"fermenters","node":"' + record.node + '","unit":"' + record.alias + '","light_state":100}');
  window.open(record.webcam_url);
 });
 createAbortElements();

 websocket.onmessage = function(evt) {
  var msg = evt.data;
  var obj = JSON.parse(msg);

  if (obj.device == "fermenters" && obj.node == record.node && obj.unit == record.alias) {
//   console.log('ws got this device ' + msg);
   record.online = obj.online;
   if (obj.online) {
    record.beeruuid = obj.beeruuid;
    record.beercode = obj.beercode;
    record.beername = obj.beername;
    record.yeast_lo = obj.yeast_lo;
    record.yeast_hi = obj.yeast_hi;
    record.air_state = obj.air_state;
    record.air_temperature = obj.air_temperature;
    record.beer_state = obj.beer_state;
    record.beer_temperature = obj.beer_temperature;
    record.chiller_state = obj.chiller_state;
    record.chiller_temperature = obj.chiller_temperature;
    if (obj.heater_state !== undefined)
     record.heater_state = obj.heater_state;
    if (obj.cooler_state !== undefined)
     record.cooler_state = obj.cooler_state;
    if (obj.fan_state !== undefined)
    record.fan_state = obj.fan_state;
    if (obj.door_address)
     record.door_state = obj.door_state;
    if (obj.light_address)
     record.light_state = obj.light_state;
    if (obj.psu_address)
     record.psu_state = obj.psu_state;
    record.mode = obj.mode;
    record.stage = obj.stage;
    record.alarm = obj.alarm;
    record.setpoint_low = obj.setpoint_low;
    record.setpoint_high = obj.setpoint_high;
    record.webcam_url = obj.webcam_url;
    record.webcam_light = obj.webcam_light;
    if (obj.profile_name) {
     record.profile_uuid = obj.profile_uuid;
     record.profile_name = obj.profile_name;
     record.profile_state = obj.profile_state;
     record.profile_percent = obj.profile_percent;
     record.profile_inittemp_high = obj.profile_inittemp_high;
     record.profile_inittemp_low = obj.profile_inittemp_low;
    } else {
     record.profile_uuid = '';
     record.profile_name = '';
     record.profile_state = '';
     record.profile_percent = 0;
    }
   }
   updateScreen();
  }
 }
});
